#ifndef paintown_gui_lineedit_h
#define paintown_gui_lineedit_h

#include <iostream>
#include <vector>

#include "gui/widget.h"
#include "gui/timer.h"

#include "util/font.h"


class keys;

class LineEdit : public Widget {
public:
	//! enumerator housing alignment positions
	enum textAlign {
		T_Left = 0,
		T_Middle,
		T_Right,
		T_Top,
		T_Bottom
	};
	//! Constructor
	LineEdit();

	//! Destructor
	~LineEdit();

	//! Set text
	void setText(const std::string & text);

	//! Get text
	const std::string & getText();

	//! Clear text
	void clearText();

	//! Set text limit (default 0, no limit)
	void setLimit(unsigned int l);

	//! Set Horizontal Alignment
	void setHorizontalAlign(const textAlign a);

	//! Set Vertical Alignment
	void setVerticalAlign(const textAlign a);

	//! Set textColor
	void setTextColor(const int color);

	//! Set textColor
	void setCursorColor(const int color);

	//! Set font
	void setFont(const Font *f);

	//! Set autoResizeable
	void setAutoResize(bool r);

	//! Set the cursor blink rate in miliseconds (default 500)
	void setCursorBlinkRate(unsigned int msecs);

	//! Update
	void logic();

	//! Draw
	void render(const Bitmap *work);
	void render(const Bitmap & work);

	//! set Focus
	void setFocused(bool focus);

	//! check Focus
	bool isFocused();

	//! Keypresses
	sigslot::slot keyPress(const keys &k);

	bool didChanged( unsigned long long & counter );

protected:
	//! Current font the label is currently using (uses systemFont by default)
	const Font *currentSetFont;

	//! Current set text
	std::string currentSetText;

	//! Horizontal Alignment (RIGHT, MIDDLE, LEFT)
	textAlign hAlignment;

	//! Additional checker for Horizontal alignment for easy adjustment
	textAlign hAlignMod;

	//! Vertical Alignment (TOP, MIDDLE, BOTTOM)
	textAlign vAlignment;

	void changed(){
		changed_ = true;
		changeCounter += 1;
	}

	void stable(){
		changed_ = false;
	}

	//! Has changed?
	bool changed_;

	//! Auto resize
	bool autoResizable;

	//! Text horizontal position
	int textX;

	//! Text vertical position
	int textY;

	//! Cursor horizontal position
	int cursorX;

	//! Cursor vertical position
	int cursorY;

	//! Cursor index relative to the string
	unsigned int cursorIndex;

	//! Text color
	int textColor;

	//! Text size Height check in case it changes
	int textSizeH;

	//! Text limit
	unsigned int limit;

	//! Cursor timer
	guiTimer cursorTime;

	//! Cursor blink rate
	unsigned int blinkRate;

	//! If the font size changes
	void fontChange();

	//! Focused?
	bool focused;

	// keeps track of changes
	unsigned long long changeCounter;
};

#endif
